<?php

namespace Nwb\Satellites;

class HttpClient
{
    public function request(string $url, array $options): bool|string
    {
        $curl = curl_init($url);
        $stderr = fopen('php://temp', 'wb');

        try {
            // Les satellites ne sont pas compatibles HTTP2
            $options[CURLOPT_HTTP_VERSION] = CURL_HTTP_VERSION_1_1;
            $options[CURLOPT_VERBOSE] = true;
            $options[CURLOPT_STDERR] = $stderr;
            $options[CURLOPT_SSL_VERIFYPEER] = false;

            curl_setopt_array($curl, $options);

            $response = curl_exec($curl);
            $errno = curl_errno($curl);
            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

            rewind($stderr);
            $log = stream_get_contents($stderr);

            if (!$response) {
                throw new SatelliteException(
                    $this->debugMessage(['reason' => 'The server dit not respond', 'curlLog' => $log])
                );
            }

            if ($httpCode !== 200 && $httpCode !== 302) {
                throw new SatelliteException(
                    $this->debugMessage(['reason' => 'HTTP error', 'curlLog' => $log, 'responseBody' => $response])
                );
            }

            if ($errno) {
                throw new SatelliteException(
                    $this->debugMessage(['reason' => 'cURL error', 'curlLog' => $log, 'curlErrno' => $errno])
                );
            }
        } finally {
            curl_close($curl);
            fclose($stderr);
        }

        return $response;
    }

    private function debugMessage(array $params): string
    {
        $message = '<h1>cURL Debug</h1>';

        if (isset($params['reason'])) {
            $message .= "<p>Reason: {$params['reason']}</p>";
        }

        $message .= '<p>Date: ' . date('Y-m-d H:i:s') . '</p>';

        if (isset($params['curlErrno'])) {
            $message .= '<h2>cURL Error</h2>';
            $message .= "<p>#{$params['curlErrno']}: " . curl_strerror($params['curlErrno']) . '</p>';
        }
        if (isset($params['curlLog'])) {
            $message .= '<h2>cURL Log</h2>';
            $message .= "<pre>{$params['curlLog']}</pre>";
        }
        if (isset($params['responseBody'])) {
            $message .= '<h2>Server response</h2>';
            $message .= "<p>{$params['responseBody']}</p>";
        }

        return $message;
    }
}
